/****************************************************************************
*                                                                           *
* File    : ShowInc.c                                                       *
*                                                                           *
* Purpose : Add-in DLL to show include from active document window          *
*                                                                           *
* History : Date       Reason                                               *
*           05-05-22  Created by timovjl extended to load local headers     *
*                     by JohnF                                              *
*                                                                           *
****************************************************************************/

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdio.h>
#include <addin.h>
#include "ShowInc.h"

// Private command ID - any number will do.
#define ID_SHOWINC  1

// Locals.
static HANDLE g_hDll = NULL;
static HWND g_hwndMain = NULL;

/****************************************************************************
*                                                                           *
* Function: DllMain                                                         *
*                                                                           *
* Purpose : DLL entry and exit procedure.                                   *
*                                                                           *
* History : Date       Reason                                               *
*            04-06-30  Created                                              *
*                                                                           *
****************************************************************************/

BOOL WINAPI DllMain(HANDLE hDLL, DWORD dwReason, LPVOID lpReserved)
{
	switch (dwReason)
	{
	case DLL_PROCESS_ATTACH:
		g_hDll = hDLL;
		return TRUE;
		
	case DLL_PROCESS_DETACH:
		return TRUE;
		
	default:
		return TRUE;
	}
}

/****************************************************************************
*                                                                           *
* Function: AddInMain                                                       *
*                                                                           *
* Purpose : Add-in entry and exit procedure.                                *
*                                                                           *
* History : Date       Reason                                               *
*            04-06-30  Created                                              *
*                                                                           *
****************************************************************************/

ADDINAPI BOOL WINAPI AddInMain(HWND hwnd, ADDIN_EVENT eEvent)
{
	switch (eEvent)
	{
	case AIE_APP_CREATE:
		{
			ADDIN_ADD_COMMAND AddCmd = {0};
			
			/* Save handle of main IDE window */
			g_hwndMain = hwnd;
			
			/* Add command button to IDE toolbar */
			AddCmd.cbSize	= sizeof(AddCmd);
			AddCmd.pszText	= "Show Include";  // Tooltip text.
			AddCmd.hIcon	= LoadImage(g_hDll, MAKEINTRESOURCE(IDR_ICON1), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR|LR_SHARED);
			AddCmd.id		= ID_SHOWINC;
			AddCmd.idMenu	= AIM_MENU_SOURCE;
			return AddIn_AddCommand(hwnd, &AddCmd);
		}

	case AIE_APP_DESTROY:
		/* Remove button from IDE toolbar */
		return AddIn_RemoveCommand(hwnd, ID_SHOWINC);

	default:
		return TRUE;
	}
}

char * GetPathName(HWND hwnd)
{
	static char s[260];
	ADDIN_DOCUMENT_INFO DocInfo;
	DocInfo.cbSize = sizeof(ADDIN_DOCUMENT_INFO);
	AddIn_GetDocumentInfo(hwnd, &DocInfo);
	char *p = strrchr(DocInfo.szFilename, '\\');
	if (p)
	{
		DocInfo.szFilename[p - DocInfo.szFilename] = '\0';
		strcpy(s, DocInfo.szFilename);
		strcat(s, "\\");
	}
	else
	{
		return NULL;
	}
	return s;
}

char * GetIncludeName(HWND hwnd, BOOL * bLocal)
{
	static char s[260];
	char szTmp[260];
	ADDIN_RANGE airRange;
	int iLine, iLen;

	if (AddIn_GetSourceSel(hwnd, &airRange))
	{
		iLine = AddIn_SourceLineFromChar(hwnd, airRange.iStartPos);
		iLen = AddIn_GetSourceLineLength(hwnd, iLine);
		if (iLen >= 256)
			return NULL;

		AddIn_GetSourceLine(hwnd, iLine, szTmp, sizeof(szTmp));
		if (!strstr(szTmp, "#include"))
			return NULL;

		char *p = strchr(szTmp, '<');	// start
		char *p1 = strchr(szTmp, '>');	// end
		if( (!p && p1) || (p && !p1) )
			return NULL;

		if (!p)
		{
			*bLocal = TRUE;
			p = strchr(szTmp, '"');	// start
			p1 = strrchr(szTmp, '"');	// end
		}
		if (!p || !p1)
			return NULL;
		else
		{
			*p1 = '\0';
			strcpy(s, p + 1);	// start
		}
	}
	return s;
}

void DoGetInclude(HWND hwnd)
{
	BOOL bFailed = TRUE;
	BOOL bIsLocal = FALSE;
	HKEY hKey;
	DWORD nLen, rType = REG_SZ;
	char *p, *p1, szLocalPath[260];
	char szTmp[1024], szMess[1024];
	char *pPath, *pFile;

	AddIn_ClearOutput(g_hwndMain);

	if(!(pFile = GetIncludeName(hwnd, &bIsLocal))){
		goto ende;
	}

	if(!(pPath = GetPathName(hwnd)))
	{
		goto ende;
	}

	if(bIsLocal)
	{
		sprintf(szMess, "%s%s", "Searching local path for ", pFile);
		AddIn_WriteOutput(g_hwndMain, szMess);
	}
	else
	{
		sprintf(szMess, "%s%s", "Searching PellesC paths for ", pFile);
		AddIn_WriteOutput(g_hwndMain, szMess);
	}

	if(bIsLocal)
	{
		AddIn_WriteOutput(g_hwndMain, pPath);
		strcpy(szLocalPath, pPath);	// path
		strcat(szLocalPath, pFile);	// plus name
		if(GetFileAttributes(szLocalPath) != -1)
		{
			bFailed = FALSE;
			sprintf(szMess, "%s%s", "Found in ", pPath);
			AddIn_WriteOutput(g_hwndMain, szMess);
			AddIn_OpenDocument(g_hwndMain, AID_SOURCE, (PCTSTR)szLocalPath);
			goto ende;
		}
		else
		{
			goto ende;
		}
	}

	nLen = sizeof(szTmp);

	// If not local search PellseC folders
	if (RegOpenKeyEx(HKEY_CURRENT_USER, "Software\\Pelle Orinius\\PellesC\\Directories", 0, KEY_READ, &hKey) != ERROR_SUCCESS)
		goto ende;

	if (RegQueryValueEx(hKey, "IncludeDirs", NULL, &rType, (unsigned char *)szTmp, &nLen) == ERROR_SUCCESS)
	{
		RegCloseKey(hKey);
		strcat(szTmp, ";"); // add an end marker
		p = szTmp;
		while (p)
		{
			p1 = strchr(p, ';');
			if (p1)
				*p1 = 0;
			else
				break;
			strcpy(szLocalPath, p);
			strcat(szLocalPath, "\\"); // path
			AddIn_WriteOutput(g_hwndMain, szLocalPath);
			strcat(szLocalPath, pFile); // plus name
			if (GetFileAttributes(szLocalPath) != -1)
			{
				bFailed = FALSE;
				sprintf(szMess, "%s%s", "Found in ", p);
				AddIn_WriteOutput(g_hwndMain, szMess);
				AddIn_OpenDocument(g_hwndMain, AID_SOURCE, (PCTSTR)szLocalPath);
				break;
			}
			p = p1 + 1; // move on to next marker ';'
		}
	}

  ende:
	
	if (bFailed == TRUE)
	{
		MessageBeep(16);
		AddIn_WriteOutput(g_hwndMain, "Not Found!");
	}
}

/****************************************************************************
*                                                                           *
* Function: AddInCommand                                                    *
*                                                                           *
* Purpose : Add-in command handler.                                         *
*                                                                           *
* History : Date       Reason                                               *
*            04-06-30  Created                                              *
*                                                                           *
****************************************************************************/
ADDINAPI void WINAPI AddInCommand(int idCmd)
{
	HWND hWnd;
	switch (idCmd)
	{
	case ID_SHOWINC:
		{
			hWnd = AddIn_GetActiveDocument(g_hwndMain);
			if (!hWnd) 
					break;	// no window, no fun
			DoGetInclude(hWnd);

		}
		break;
	}
}

